@@ -1,50 +1,70 @@
-### Copyright (C) 2002-2006 Stephen Kennedy <stevek@gnome.org>
-### Copyright (C) 2009-2011 Kai Willadsen <kai.willadsen@gmail.com>
-
-### This program is free software; you can redistribute it and/or modify
-### it under the terms of the GNU General Public License as published by
-### the Free Software Foundation; either version 2 of the License, or
-### (at your option) any later version.
-
-### This program is distributed in the hope that it will be useful,
-### but WITHOUT ANY WARRANTY; without even the implied warranty of
-### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-### GNU General Public License for more details.
-
-### You should have received a copy of the GNU General Public License
-### along with this program; if not, write to the Free Software
-### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
-### USA.
+# Copyright (C) 2002-2006 Stephen Kennedy <stevek@gnome.org>
+# Copyright (C) 2009-2013 Kai Willadsen <kai.willadsen@gmail.com>
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 2 of the License, or (at
+# your option) any later version.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import unicode_literals
 
 import sys
 from gettext import gettext as _
 
-from .util import sourceviewer
+from gi.repository import Gio
+from gi.repository import GLib
+from gi.repository import GObject
+from gi.repository import GtkSource
 
-from .util.compat import text_type
+from meld.util.compat import text_type
 
 
-class MeldBuffer(sourceviewer.srcviewer.GtkTextBuffer):
+class MeldBuffer(GtkSource.Buffer):
 
     __gtype_name__ = "MeldBuffer"
 
     def __init__(self, filename=None):
-        sourceviewer.srcviewer.GtkTextBuffer.__init__(self)
+        GtkSource.Buffer.__init__(self)
         self.data = MeldBufferData(filename)
+        self.user_action_count = 0
+
+    def do_begin_user_action(self, *args):
+        self.user_action_count += 1
+
+    def do_end_user_action(self, *args):
+        self.user_action_count -= 1
+
+    def do_apply_tag(self, tag, start, end):
+        # Filthy, evil, horrible hack. What we're doing here is trying to
+        # figure out if a tag apply has come from a paste action, in which
+        # case GtkTextBuffer will 'helpfully' apply the existing tags in the
+        # copied selection. There appears to be no way to override this
+        # behaviour, or to hook in to the necessary paste mechanics to just
+        # request that we only get plain text or something. We're abusing the
+        # user_action notion here, because we only apply the tags we actually
+        # want in a callback.
+        if tag.props.name == 'inline' and self.user_action_count > 0:
+            return
+        return GtkSource.Buffer.do_apply_tag(self, tag, start, end)
 
     def reset_buffer(self, filename):
         """Clear the contents of the buffer and reset its metadata"""
         self.delete(*self.get_bounds())
-
-        new_data = MeldBufferData(filename)
-        if self.data.filename == filename:
-            new_data.label = self.data.label
-        self.data = new_data
+        label = self.data.label if self.data.filename == filename else filename
+        self.data.reset()
+        self.data.filename = filename
+        self.data.label = label
 
     def get_iter_at_line_or_eof(self, line):
-        """Return a gtk.TextIter at the given line, or the end of the buffer.
+        """Return a Gtk.TextIter at the given line, or the end of the buffer.
 
         This method is like get_iter_at_line, but if asked for a position past
         the end of the buffer, this returns the end of the buffer; the
@@ -60,7 +80,7 @@ class MeldBuffer(sourceviewer.srcviewer.GtkTextBuffer):
 
         This method is like insert, but if asked to insert something past the
         last line in the buffer, this will insert at the end, and will add a
-        linebreak before the inserted text. The last line in a gtk.TextBuffer
+        linebreak before the inserted text. The last line in a Gtk.TextBuffer
         is guaranteed never to have a newline, so we need to handle this.
         """
         if line >= self.get_line_count():
@@ -72,37 +92,101 @@ class MeldBuffer(sourceviewer.srcviewer.GtkTextBuffer):
         return it
 
 
-class MeldBufferData(object):
+class MeldBufferData(GObject.GObject):
+
+    __gsignals__ = {
+        str('file-changed'): (GObject.SignalFlags.RUN_FIRST, None, ()),
+    }
 
     def __init__(self, filename=None):
+        GObject.GObject.__init__(self)
+        self.reset()
+        self._label = self.filename = filename
+
+    def reset(self):
         self.modified = False
         self.writable = True
         self.editable = True
-        self.filename = filename
+        self._monitor = None
+        self._mtime = None
+        self._disk_mtime = None
+        self.filename = None
         self.savefile = None
-        self._label = filename
+        self._label = None
         self.encoding = None
         self.newlines = None
 
-    def get_label(self):
+    def __del__(self):
+        self._disconnect_monitor()
+
+    @property
+    def label(self):
         #TRANSLATORS: This is the label of a new, currently-unnamed file.
         return self._label or _("<unnamed>")
 
-    def set_label(self, value):
+    @label.setter
+    def label(self, value):
         self._label = value
 
-    label = property(get_label, set_label)
+    def _connect_monitor(self):
+        if self._filename:
+            monitor = Gio.File.new_for_path(self._filename).monitor_file(
+                Gio.FileMonitorFlags.NONE, None)
+            handler_id = monitor.connect('changed', self._handle_file_change)
+            self._monitor = monitor, handler_id
+
+    def _disconnect_monitor(self):
+        if self._monitor:
+            monitor, handler_id = self._monitor
+            monitor.disconnect(handler_id)
+            monitor.cancel()
+
+    def _query_mtime(self, gfile):
+        try:
+            time_query = ",".join((Gio.FILE_ATTRIBUTE_TIME_MODIFIED,
+                                   Gio.FILE_ATTRIBUTE_TIME_MODIFIED_USEC))
+            info = gfile.query_info(time_query, 0, None)
+        except GLib.GError:
+            return None
+        mtime = info.get_modification_time()
+        return (mtime.tv_sec, mtime.tv_usec)
+
+    def _handle_file_change(self, monitor, f, other_file, event_type):
+        mtime = self._query_mtime(f)
+        if self._disk_mtime and mtime > self._disk_mtime:
+            self.emit('file-changed')
+        self._disk_mtime = mtime
+
+    @property
+    def filename(self):
+        return self._filename
+
+    @filename.setter
+    def filename(self, value):
+        self._disconnect_monitor()
+        self._filename = value
+        self.update_mtime()
+        self._connect_monitor()
+
+    def update_mtime(self):
+        if self._filename:
+            gfile = Gio.File.new_for_path(self._filename)
+            self._disk_mtime = self._query_mtime(gfile)
+            self._mtime = self._disk_mtime
+
+    def current_on_disk(self):
+        return self._mtime == self._disk_mtime
 
 
 class BufferLines(object):
-    """gtk.TextBuffer shim with line-based access and optional filtering
+    """Gtk.TextBuffer shim with line-based access and optional filtering
 
-    This class allows a gtk.TextBuffer to be treated as a list of lines of
+    This class allows a Gtk.TextBuffer to be treated as a list of lines of
     possibly-filtered text. If no filter is given, the raw output from the
-    gtk.TextBuffer is used.
+    Gtk.TextBuffer is used.
 
     The logic here (and in places in FileDiff) requires that Python's
-    unicode splitlines() implementation and gtk.TextBuffer agree on where
+    unicode splitlines() implementation and Gtk.TextBuffer agree on where
     linebreaks occur. Happily, this is usually the case.
     """
 
@@ -127,7 +211,7 @@ class BufferLines(object):
             lines = filter_txt.splitlines()
             ends = filter_txt.splitlines(True)
 
-            # The last line in a gtk.TextBuffer is guaranteed never to end in a
+            # The last line in a Gtk.TextBuffer is guaranteed never to end in a
             # newline. As splitlines() discards an empty line at the end, we
             # need to artificially add a line if the requested slice is past
             # the end of the buffer, and the last line in the slice ended in a
