@@ -1,21 +1,19 @@
-### Copyright (C) 2002-2006 Stephen Kennedy <stevek@gnome.org>
-### Copyright (C) 2009 Vincent Legoll <vincent.legoll@gmail.com>
-### Copyright (C) 2012 Kai Willadsen <kai.willadsen@gmail.com>
-
-### This program is free software; you can redistribute it and/or modify
-### it under the terms of the GNU General Public License as published by
-### the Free Software Foundation; either version 2 of the License, or
-### (at your option) any later version.
-
-### This program is distributed in the hope that it will be useful,
-### but WITHOUT ANY WARRANTY; without even the implied warranty of
-### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-### GNU General Public License for more details.
-
-### You should have received a copy of the GNU General Public License
-### along with this program; if not, write to the Free Software
-### Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
-### USA.
+# Copyright (C) 2002-2006 Stephen Kennedy <stevek@gnome.org>
+# Copyright (C) 2009 Vincent Legoll <vincent.legoll@gmail.com>
+# Copyright (C) 2012-2013 Kai Willadsen <kai.willadsen@gmail.com>
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 2 of the License, or (at
+# your option) any later version.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """Module of commonly used helper classes and functions
 """
@@ -27,13 +25,10 @@ import shutil
 import re
 import subprocess
 
-import gio
-import gobject
-import gtk
+from gi.repository import GObject
+from gi.repository import Gtk
 
 
-whitespace_re = re.compile(r"\s")
-
 if os.name != "nt":
     from select import select
 else:
@@ -44,22 +39,41 @@ else:
         return rlist, wlist, xlist
 
 
-def shelljoin( command ):
-    def quote(s):
-        return ((whitespace_re.search(s) is None) and s or ('"%s"' % s))
-    return " ".join( [ quote(x) for x in command ] )
+def error_dialog(primary, secondary, parent=None, messagetype=Gtk.MessageType.ERROR):
+    """A common error dialog handler for Meld
 
-def run_dialog( text, parent=None, messagetype=gtk.MESSAGE_WARNING, buttonstype=gtk.BUTTONS_OK, extrabuttons=()):
+    This should only ever be used as a last resort, and for errors that
+    a user is unlikely to encounter. If you're tempted to use this,
+    think twice.
+
+    Primary must be plain text. Secondary must be valid markup.
+    """
+    if not parent:
+        from meld.meldapp import app
+        parent = app.window.widget
+
+    dialog = Gtk.MessageDialog(
+        parent=parent,
+        flags=Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+        type=messagetype,
+        buttons=Gtk.ButtonsType.CLOSE,
+        message_format=primary)
+    dialog.format_secondary_markup(secondary)
+    dialog.run()
+    dialog.destroy()
+
+
+def run_dialog( text, parent=None, messagetype=Gtk.MessageType.WARNING, buttonstype=Gtk.ButtonsType.OK, extrabuttons=()):
     """Run a dialog with text 'text'.
        Extra buttons are passed as tuples of (button label, response id).
     """
-    escaped = gobject.markup_escape_text(text)
-    d = gtk.MessageDialog(None,
-        gtk.DIALOG_DESTROY_WITH_PARENT,
+    escaped = GObject.markup_escape_text(text)
+    d = Gtk.MessageDialog(None,
+        Gtk.DialogFlags.DESTROY_WITH_PARENT,
         messagetype,
         buttonstype,
         '<span weight="bold" size="larger">%s</span>' % escaped)
-    if parent and isinstance(parent, gtk.Window):
+    if parent and isinstance(parent, Gtk.Window):
         d.set_transient_for(parent.widget.get_toplevel())
     for b,rid in extrabuttons:
         d.add_button(b, rid)
@@ -74,46 +88,38 @@ def run_dialog( text, parent=None, messagetype=gtk.MESSAGE_WARNING, buttonstype=
     d.destroy()
     return ret
 
-def open_uri(uri, timestamp=0):
-    try:
-        gtk.show_uri(gtk.gdk.screen_get_default(), uri, timestamp)
-    except gio.Error:
-        if uri.startswith("http://"):
-            import webbrowser
-            webbrowser.open_new_tab(uri)
-        else:
-            # Unhandled URI
-            pass
 
 # Taken from epiphany
 def position_menu_under_widget(menu, widget):
-    container = widget.get_ancestor(gtk.Container)
+    container = widget.get_ancestor(Gtk.Container)
 
-    widget_width, widget_height = widget.size_request()
-    menu_width, menu_height = menu.size_request()
+    widget_width = widget.get_allocation().width
+    menu_width = menu.get_allocation().width
+    menu_height = menu.get_allocation().height
 
     screen = menu.get_screen()
-    monitor_num = screen.get_monitor_at_window(widget.window)
+    monitor_num = screen.get_monitor_at_window(widget.get_window())
     if monitor_num < 0:
         monitor_num = 0
     monitor = screen.get_monitor_geometry(monitor_num)
 
-    x, y = widget.window.get_origin()
-    if widget.flags() & gtk.NO_WINDOW:
-        x += widget.allocation.x
-        y += widget.allocation.y
+    unused, x, y = widget.get_window().get_origin()
+    allocation = widget.get_allocation()
+    if not widget.get_has_window():
+        x += allocation.x
+        y += allocation.y
 
-    if container.get_direction() == gtk.TEXT_DIR_LTR:
-        x += widget.allocation.width - widget_width
+    if container.get_direction() == Gtk.TextDirection.LTR:
+        x += allocation.width - widget_width
     else:
         x += widget_width - menu_width
 
-    if (y + widget.allocation.height + menu_height) <= monitor.y + monitor.height:
-        y += widget.allocation.height
+    if (y + allocation.height + menu_height) <= monitor.y + monitor.height:
+        y += allocation.height
     elif (y - menu_height) >= monitor.y:
         y -= menu_height
-    elif monitor.y + monitor.height - (y + widget.allocation.height) > y:
-        y += widget.allocation.height
+    elif monitor.y + monitor.height - (y + allocation.height) > y:
+        y += allocation.height
     else:
         y -= menu_height
 
@@ -121,11 +127,11 @@ def position_menu_under_widget(menu, widget):
 
 def make_tool_button_widget(label):
     """Make a GtkToolButton label-widget suggestive of a menu dropdown"""
-    arrow = gtk.Arrow(gtk.ARROW_DOWN, gtk.SHADOW_NONE)
-    label = gtk.Label(label)
-    hbox = gtk.HBox(spacing=3)
-    hbox.pack_end(arrow)
-    hbox.pack_end(label)
+    arrow = Gtk.Arrow(Gtk.ArrowType.DOWN, Gtk.ShadowType.NONE)
+    label = Gtk.Label(label=label)
+    hbox = Gtk.HBox(spacing=3)
+    hbox.pack_end(arrow, True, True, 0)
+    hbox.pack_end(label, True, True, 0)
     hbox.show_all()
     return hbox
 
