@@ -1,19 +1,17 @@
-# Copyright (C) 2012 Kai Willadsen <kai.willadsen@gmail.com>
-
-# This program is free software; you can redistribute it and/or modify
+# Copyright (C) 2012-2013 Kai Willadsen <kai.willadsen@gmail.com>
+#
+# This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
+# the Free Software Foundation, either version 2 of the License, or (at
+# your option) any later version.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+#
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
-# USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """
 Recent files integration for Meld's multi-element comparisons
@@ -26,18 +24,22 @@ single-file registers for multi-file comparisons, and tell the recent files
 infrastructure that that's actually what we opened.
 """
 
-import ConfigParser
+try:
+    # py3k
+    import configparser
+except ImportError:
+    import ConfigParser as configparser
 import os
 import sys
 import tempfile
 
 from gettext import gettext as _
 
-import gio
-import glib
-import gtk
+from gi.repository import Gio
+from gi.repository import GLib
+from gi.repository import Gtk
 
-from . import misc
+import meld.misc
 
 
 TYPE_FILE = "File"
@@ -49,23 +51,19 @@ COMPARISON_TYPES = (TYPE_FILE, TYPE_FOLDER, TYPE_VC, TYPE_MERGE)
 
 class RecentFiles(object):
 
-    recent_dirname = "Meld" if sys.platform == "win32" else "meld"
-    recent_path = os.path.join(glib.get_user_data_dir(), recent_dirname)
+    recent_path = os.path.join(GLib.get_user_data_dir(), "meld")
     recent_path = recent_path.decode('utf8')
     recent_suffix = ".meldcmp"
 
     # Recent data
     app_name = "Meld"
-    app_exec = "meld"
 
-    def __init__(self, exec_path=None):
-        self.recent_manager = gtk.recent_manager_get_default()
-        self.recent_filter = gtk.RecentFilter()
+    def __init__(self):
+        self.recent_manager = Gtk.RecentManager.get_default()
+        self.recent_filter = Gtk.RecentFilter()
         self.recent_filter.add_application(self.app_name)
         self._stored_comparisons = []
-        # Should be argv[0] to support roundtripping in uninstalled use
-        if exec_path:
-            self.app_exec = os.path.abspath(exec_path)
+        self.app_exec = os.path.abspath(sys.argv[0])
 
         if not os.path.exists(self.recent_path):
             os.makedirs(self.recent_path)
@@ -91,13 +89,13 @@ class RecentFiles(object):
         # the corresponding comparison file
         comparison_key = (comp_type, tuple(paths))
         if comparison_key in self._stored_comparisons:
-            gio_file = gio.File(uri=self._stored_comparisons[comparison_key])
+            gio_file = Gio.File.new_for_uri(self._stored_comparisons[comparison_key])
         else:
             recent_path = self._write_recent_file(comp_type, paths)
-            gio_file = gio.File(path=recent_path)
+            gio_file = Gio.File.new_for_path(recent_path)
 
         if len(paths) > 1:
-            display_name = " : ".join(misc.shorten_names(*paths))
+            display_name = " : ".join(meld.misc.shorten_names(*paths))
         else:
             display_path = paths[0]
             userhome = os.path.expanduser("~")
@@ -108,14 +106,13 @@ class RecentFiles(object):
         # FIXME: Should this be translatable? It's not actually used anywhere.
         description = "%s comparison\n%s" % (comp_type, ", ".join(paths))
 
-        recent_metadata = {
-            "mime_type": "application/x-meld-comparison",
-            "app_name": self.app_name,
-            "app_exec": "%s --comparison-file %%u" % self.app_exec,
-            "display_name": display_name.encode('utf8'),
-            "description": description.encode('utf8'),
-            "is_private": True,
-        }
+        recent_metadata = Gtk.RecentData()
+        recent_metadata.mime_type = "application/x-meld-comparison"
+        recent_metadata.app_name = self.app_name
+        recent_metadata.app_exec = "%s --comparison-file %%u" % self.app_exec
+        recent_metadata.display_name = display_name.encode('utf8')
+        recent_metadata.description = description.encode('utf8')
+        recent_metadata.is_private = True
         self.recent_manager.add_full(gio_file.get_uri(), recent_metadata)
 
     def read(self, uri):
@@ -124,18 +121,18 @@ class RecentFiles(object):
         Returns the comparison type, the paths involved and the comparison
         flags.
         """
-        gio_file = gio.File(uri=uri)
+        gio_file = Gio.File.new_for_uri(uri)
         path = gio_file.get_path()
-        if not gio_file.query_exists() or not path:
+        if not gio_file.query_exists(None) or not path:
             raise IOError("File does not exist")
 
         try:
-            config = ConfigParser.RawConfigParser()
+            config = configparser.RawConfigParser()
             config.read(path)
             assert (config.has_section("Comparison") and
                     config.has_option("Comparison", "type") and
                     config.has_option("Comparison", "paths"))
-        except (ConfigParser.Error, AssertionError):
+        except (configparser.Error, AssertionError):
             raise ValueError("Invalid recent comparison file")
 
         comp_type = config.get("Comparison", "type")
@@ -148,13 +145,13 @@ class RecentFiles(object):
         return comp_type, paths, flags
 
     def _write_recent_file(self, comp_type, paths):
-        # TODO: Use GKeyFile instead, and return a gio.File. This is why we're
+        # TODO: Use GKeyFile instead, and return a Gio.File. This is why we're
         # using ';' to join comparison paths.
         with tempfile.NamedTemporaryFile(prefix='recent-',
                                          suffix=self.recent_suffix,
                                          dir=self.recent_path,
                                          delete=False) as f:
-            config = ConfigParser.RawConfigParser()
+            config = configparser.RawConfigParser()
             config.add_section("Comparison")
             config.set("Comparison", "type", comp_type)
             config.set("Comparison", "paths", ";".join(paths))
@@ -174,7 +171,7 @@ class RecentFiles(object):
 
         # Remove any comparison files that are not listed by RecentManager
         item_uris = [item.get_uri() for item in meld_items]
-        item_paths = [gio.File(uri=uri).get_path() for uri in item_uris]
+        item_paths = [Gio.File.new_for_uri(uri).get_path() for uri in item_uris]
         stored = [p for p in os.listdir(self.recent_path)
                   if p.endswith(self.recent_suffix)]
         for path in stored:
@@ -196,20 +193,26 @@ class RecentFiles(object):
             self._stored_comparisons[comp[:2]] = uri
 
     def _filter_items(self, recent_filter, items):
-        getters = {gtk.RECENT_FILTER_URI: "uri",
-                   gtk.RECENT_FILTER_DISPLAY_NAME: "display_name",
-                   gtk.RECENT_FILTER_MIME_TYPE: "mime_type",
-                   gtk.RECENT_FILTER_APPLICATION: "applications",
-                   gtk.RECENT_FILTER_GROUP: "groups",
-                   gtk.RECENT_FILTER_AGE: "age"}
+        getters = {Gtk.RecentFilterFlags.URI: "uri",
+                   Gtk.RecentFilterFlags.DISPLAY_NAME: "display_name",
+                   Gtk.RecentFilterFlags.MIME_TYPE: "mime_type",
+                   Gtk.RecentFilterFlags.APPLICATION: "applications",
+                   Gtk.RecentFilterFlags.GROUP: "groups",
+                   Gtk.RecentFilterFlags.AGE: "age"}
         needed = recent_filter.get_needed()
         attrs = [v for k, v in getters.iteritems() if needed & k]
 
         filtered_items = []
         for i in items:
-            filter_info = {}
+            filter_data = {}
             for attr in attrs:
-                filter_info[attr] = getattr(i, "get_" + attr)()
+                filter_data[attr] = getattr(i, "get_" + attr)()
+            filter_info = Gtk.RecentFilterInfo()
+            for f, v in filter_data.iteritems():
+                # https://bugzilla.gnome.org/show_bug.cgi?id=695970
+                if isinstance(v, list):
+                    continue
+                setattr(filter_info, f, v)
             if recent_filter.filter(filter_info):
                 filtered_items.append(i)
         return filtered_items
@@ -223,6 +226,4 @@ class RecentFiles(object):
         return "\n".join(descriptions)
 
 
-if __name__ == "__main__":
-    recent = RecentFiles()
-    print recent
+recent_comparisons = RecentFiles()
